<?php 
	namespace Envia\Templates\Loader;

	defined( 'ABSPATH' ) || exit;

class Envia_Templates {
	public static function print_envia_page() {
		if ( is_admin() ) {
			if ( current_user_can( 'manage_options' ) && current_user_can( 'manage_woocommerce' ) ) {
				$companyId = get_option( 'woocommerce_envia_shipping_settings' )['company'];
				$storeId = get_option( 'woocommerce_envia_shipping_settings' )['shop'];
				$storeUrl = site_url();
				if ( ! is_null( $storeId ) && ! is_null( $storeUrl ) ) {
					$args = array(
						'status' => 'true',
						'hash' => base64_encode( $storeUrl . ':' . $companyId . ':' . $storeId ),
						'enviaUrl' => 'https://shipping.envia.com/ecommerce',
					);
				} else {
					$args = array( 
						'status' => 'false',
						'message' => 'Start to use Envia.com in your store admin.',
					);
				}
			} else {
				$args = array( 
					'status' => 'unauthorized',
					'message' => 'Your user account does not have permission to display this page.',
				);
			}
			self::envia_template_part('envia', 'display', $args);
		}
	}

	public static function envia_template_part( $slug, $name = null, $args = array() ) {
		do_action( "get_template_part_{$slug}", $slug, $name, $args );

		$templates = array();
		$name      = (string) $name;
		if ( '' !== $name ) {
			$templates[] = "{$slug}-{$name}.php";
		}

		$templates[] = "{$slug}.php";
		self::envia_template_loader( $templates, $args );
	}

	public static function envia_template_loader( $templates, $args ) {
		$toLoad = null;
		foreach ( $templates as $template ) {
			if ( file_exists( MDABSPATH . '/templates/' . $template ) ) {
				$toLoad = MDABSPATH . '/templates/' . $template;
			}
		}
		if ( ! is_null( $toLoad ) ) {
			load_template( $toLoad, false, $args );
		}

		$css = array(
			'ordersAdmin' => array( 
				'file' => 'orders-admin',
				'version' => '1.4',
			),
		);
		self::orders_css_loader( $css );
	}
	
	public static function orders_css_loader( $files = array() ) {
		if ( 0 == count( $files ) ) {
						return 0;
		}
		foreach ( $files as $handle => $css ) {
			if ( file_exists( MDABSPATH . '/admin/css/envia-' . $css['file'] . '.css' ) ) {
				$toLoad = plugins_url( '../admin/css/envia-' . $css['file'] . '.css', __FILE__ );
				wp_enqueue_style( $handle, $toLoad, array(), $css['version'], false );
			}
		}
	}

	public static function add_submenu_option() {
		add_submenu_page(
			'woocommerce',
			__( 'Envia.com', 'envia'),
			__( 'Envia.com', 'envia'),
			'manage_woocommerce',
			'envia-order-manager',
			array( __CLASS__, 'print_envia_page' )
		);
	}
}
