<?php
add_action( 'wp_ajax_nopriv_envia_cities', 'getcities' );
add_action( 'wp_ajax_envia_cities', 'getcities' );
function getcities() {
	try {
		$country = isset( $_GET['country'] ) ? sanitize_text_field( $_GET['country'] ) : null;
		$state   = isset( $_GET['state'] ) ? sanitize_text_field( $_GET['state'] ) : null;
		if ( ! is_null( $country ) && ! is_null( $state ) ) {
			$url    = "https://geocodes.envia.com/list/localities/{$country}/{$state}";
			$cities = requests_process( 'GET', $url, null, null );
			header( 'HTTP/1.1 200 OK' );
			echo json_encode( $cities, JSON_PRETTY_PRINT );
		}
	} catch ( Exception $err ) {
		throw $err;
	}
	die();
}

add_action( 'wp_ajax_envia_forms', 'getForm' );
function getForm() {
	try {
		$country = isset( $_GET['country'] ) ? sanitize_text_field( $_GET['country'] ) : null;
		if ( ! is_null( $country ) ) {
			$url         = ENVIA_QUERIES_HOSTNAME . "/generic-form?country_code={$country}&form=address_info";
			$countryForm = requests_process( 'GET', $url, null, null );
			$states      = getStates( $country );
			header( 'HTTP/1.1 200 OK' );
			$response = array(
				'form'   => $countryForm,
				'states' => $states,
			);
			echo json_encode( $response, JSON_PRETTY_PRINT );
		}
	} catch ( Exception $err ) {
		echo json_encode(
			array(
				'status'  => 'error',
				'message' => $err->getMessage(),
				'code'    => $err->getCode(),
			)
		);
	}
	die();
}

function getStates( $country ) {
	try {
		$country = isset( $_GET['country'] ) ? sanitize_text_field( $_GET['country'] ) : $country;
		if ( ! is_null( $country ) ) {
			$url    = "https://queries.envia.com/state?country_code={$country}";
			$states = requests_process( 'GET', $url, null, null );
			if ( isset( $states['data'] ) ) {
				$statesAux = array();
				foreach ( $states['data'] as $state ) {
					$statesAux[ $state['code_2_digits'] ] = $state['name'];
				}
				$states = $statesAux;
			}
			return $states;
		}
	} catch ( Exception $err ) {
		throw $err;
	}
}

add_action( 'wp_ajax_envia_save', 'saveOriginAddress' );
function saveOriginAddress() {
	try {
		if ( isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( $_REQUEST['_wpnonce'] ), 'my-nonce' ) ) {
			$payload = isset( $_POST['form'] ) ? json_decode( str_replace( '\\', '', sanitize_text_field( $_POST['form'] ) ), true ) : null;
			$payload = recursive_sanitize_text_field( $payload );
		}
		if ( ! is_null( $payload ) ) {
			$payload['type']        = 1;
			$payload['category_id'] = 1;
			$token                  = get_option( 'woocommerce_envia_shipping_settings' )['token'];
			if ( ! $token ) {
				throw new Exception( 'You need an API Key for start to save addresses, connect your store with Envia.com', 401 );
			}
			$url     = ENVIA_QUERIES_HOSTNAME . '/user-address';
			$address = requests_process( 'POST', $url, $token, $payload );
			if ( isset( $address['id'] ) ) {
				$shopId          = get_option( 'woocommerce_envia_shipping_settings' )['shop'];
				$url             = ENVIA_QUERIES_HOSTNAME . '/shop-default-address/' . $shopId;
				$payload         = array( 'address_id' => strval( $address['id'] ) );
				$setAddressStore = requests_process( 'POST', $url, $token, $payload );
				if ( $setAddressStore['shop_id'] ) {
					echo json_encode(
						array(
							'status' => 'success',
							'code'   => '200',
						),
						JSON_PRETTY_PRINT
					);
					header( 'HTTP/1.1 200 Success' );
				} else {
					throw new Exception( 'Something went wrong, try again', 400 );
				}
			} else {
				throw new Exception( 'Something went wrong, try again', 400 );
			}
		} else {
			throw new Exception( 'Something went wrong, try again', 401 );
		}
	} catch ( Exception $err ) {
		header( "HTTP/1.1 {$err->getCode()} Error" );
		echo json_encode(
			array(
				'status'  => 'error',
				'message' => $err->getMessage(),
				'code'    => $err->getCode(),
			)
		);
	}
	die();
}

function recursive_sanitize_text_field( $array ) {
	foreach ( $array as $key => &$value ) {
		if ( is_array( $value ) ) {
			$value = recursive_sanitize_text_field( $value );
		} else {
			$value = sanitize_text_field( $value );
		}
	}
	return $array;
}

add_action( 'wp_ajax_nopriv_set_pick', 'pickDestination' );
add_action( 'wp_ajax_set_pick', 'pickDestination' );
function pickDestination() {
	try {
		if ( isset( $_REQUEST['_wpnonce'] ) && wp_verify_nonce( sanitize_text_field( $_REQUEST['_wpnonce'] ), 'my-nonce' ) ) {
			$optionData = isset( $_POST['data'] ) ? json_decode( str_replace( '\\', '', sanitize_text_field( $_POST['data'] ) ), true ) : null;
			$optionData = recursive_sanitize_text_field( $optionData );
			$customerInfo = isset( $_POST['customer'] ) ? json_decode( str_replace( '\\', '', sanitize_text_field( $_POST['customer'] ) ), true ) : null;
			$customerInfo = recursive_sanitize_text_field( $customerInfo );
			if ( 'envia' == $optionData['method'] ) {
				WC()->session->__unset( 'branch_selected' );
				if ( ! is_null( $optionData['branchCode'] ) && '' != $optionData['branchCode'] ) {
					if ( ! isset( WC()->session->branch_selected ) && ! isset( WC()->session->last_customer_shipping_address ) ) { 
						save_customer_shipping_address( $customerInfo );
					}
					WC()->session->set( 'branch_selected', $optionData['branchCode'] );
					$response = set_shipping_address( $optionData, null, $customerInfo );
					WC()->session->set( 'changed_address', true );
				} else if ( isset( WC()->session->last_customer_shipping_address ) ) {
						$address = WC()->session->last_customer_shipping_address;
						$response = set_shipping_address( null, $address, $customerInfo );
						WC()->session->__unset( 'last_customer_shipping_address' );
						WC()->session->__unset( 'branch_selected' );
						WC()->session->set( 'changed_address', true );
				} else {
					$response = array( 'status' => 'nothing changed' );
				}
			}
			echo json_encode(
				array(
					'status'  => $response['status'],
					'message' => 'changed shipping address',
					'code'    => 200,
					'whereIs' => WC()->session->get( 'rates_in' ),
					'address' => WC()->customer->get_shipping(),
				)
			);
			header( 'HTTP/1.1 200 Success' );
		}
	} catch ( Exception $err ) {
		header( "HTTP/1.1 {$err->getCode()} Error" );
		echo json_encode(
			array(
				'status'  => 'error',
				'message' => $err->getMessage(),
				'code'    => $err->getCode(),
			)
		);
	}
	die();
}

function save_customer_shipping_address( $customer = null ) {
	$beforePickSelected = array(
		'firstName' => '' != $customer['shipping_first_name'] ? $customer['shipping_first_name'] : $customer['shipping_last_name'],
		'lastName' => '' !=  $customer['shipping_last_name'] ? $customer['shipping_last_name'] : $customer['billing_last_name'],
		'company' => '' !=  $customer['shipping_company'] ? $customer['shipping_company'] : $customer['billing_company'],
		'address' => '' != WC()->customer->get_shipping_address_1() ? WC()->customer->get_shipping_address_1() : WC()->customer->get_billing_address_1(),
		'number' => '' != WC()->customer->get_shipping_address_2() ? WC()->customer->get_shipping_address_2() : WC()->customer->get_billing_address_2(),
		'city' => '' != WC()->customer->get_shipping_city() ? WC()->customer->get_shipping_city() : WC()->customer->get_billing_city(),
		'zipcode' => '' != WC()->customer->get_shipping_postcode() ? WC()->customer->get_shipping_postcode() : WC()->customer->get_billing_postcode(),
		'country'  => '' != WC()->customer->get_shipping_country() ? WC()->customer->get_shipping_country() : WC()->customer->get_billing_country(),
	);
	WC()->session->set( 'last_customer_shipping_address', $beforePickSelected );
}

/**
 *Set shipping address receive $option when is a branch address, $address for restore the first quote address and $customer always contains saved information about the customer. 
 * 
 */
function set_shipping_address( $option = null, $address = null, $customer = null ) {
	if ( ! is_null( $customer ) ) {
		if ( ! is_null( $address ) ) {		
			$shippingFirstName = $address['firstName']; 
			$shippingLastName = $address['lastName']; 
			$shippingCompany = $address['company']; 
		} else {
			$shippingFirstName = '' != $customer['shipping_first_name'] ? $customer['shipping_first_name'] : $customer['billing_first_name'];
			$shippingLastName = '' !=  $customer['shipping_last_name'] ? $customer['shipping_last_name'] : $customer['billing_last_name'];
			$shippingCompany = '' !=  $customer['shipping_company'] ? $customer['shipping_company'] : $customer['billing_company'];
		}
	}
	if ( is_null ( $address ) ) {
		$session = WC()->session->get('shipping_for_package_0');
		$rates = $session['rates'];
		$chosenOption = $option['option'];
		$rateSelected = array_key_exists( $option['option'], $rates )  ? $rates[ $chosenOption ] : null;
		if ( ! is_null( $rateSelected ) && ! is_null( $option['branchCode'] ) ) {
			if ( 'custom' == $option['viewMode'] ) {
				foreach ( $rateSelected->meta_data['branches'] as $branch ) {
					if ( $option['branchCode'] == $branch['branch_code'] ) {
						$address = $branch['address'];
					}
				}
			}
			if ( 'standard' == $option['viewMode'] ) { 
				$address = $rateSelected->meta_data['branchAddress'];
			}
		}
	}
	WC()->customer->set_billing_first_name( wc_clean( $customer['billing_first_name'] ) );
	WC()->customer->set_billing_last_name( wc_clean( $customer['billing_last_name'] ) );
	WC()->customer->set_billing_company( wc_clean( $customer['billing_company'] ) );
	WC()->customer->set_billing_phone( wc_clean( $customer['billing_phone'] ) );
	WC()->customer->set_billing_email( wc_clean( $customer['billing_email'] ) );
	WC()->customer->set_shipping_first_name( wc_clean( $shippingFirstName ) );
	WC()->customer->set_shipping_last_name( wc_clean( $shippingLastName ) );
	WC()->customer->set_shipping_company( wc_clean( $shippingCompany ) );
	WC()->customer->set_shipping_address_1( wc_clean( $address['address'] ) );
	WC()->customer->set_shipping_address_2( wc_clean( $address['number'] ) );
	WC()->customer->set_shipping_city( wc_clean( $address['city'] ) );
	// WC()->customer->set_shipping_state( wc_clean( $address['locality']; ) );
	WC()->customer->set_shipping_postcode( wc_clean( $address['zipcode'] ) );
	WC()->customer->set_shipping_country( wc_clean( $address['country'] ) );
	//add_filter( 'woocommerce_ship_to_different_address_checked', '__return_true' );
	return array( 'status' => 'success' );
}

//Shipping form is always open
if ( ! is_null( get_option( 'envia_oauth_connection' ) ) && 'true' == get_option( 'envia_oauth_connection' ) ) {
	if ( 'billing' == get_option( 'woocommerce_ship_to_destination' ) && 'yes' == get_option( 'woocommerce_envia_shipping_settings' )['pickUpDestination'] ) {
		add_filter( 'woocommerce_ship_to_different_address_checked', '__return_true' );
	}
}
