<?php
/**
 * Plugin Name: Envia Shipping and Fulfillment
 * Plugin URI:  https://woocommerce.com/es-es/products/envia-shipping-and-fulfillment/
 * Version: 4.2.5
 * Description: The Best Shipping Solution for your business. Connect to 150+ couriers worldwide and get over 70% discounts for domestic and international shipments!
 * Author: Tendecys Innovations
 * Author URI: https://tendencys.com
 * Requires at least: 6.3
 * Requires PHP: 7.4
 * Text Domain: Envia Shipping and Fulfillment
 * License: GPLv2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Woo: 18734001052053:dc29297b72b473f0e1f97e671bb3f370
 */
ob_start();
define( 'VERSION', '4.2.5' );
define( 'MAINDIR', basename( __DIR__ ) );
define( 'MDABSPATH', __DIR__ );
define( 'MAINFILE', basename( __DIR__ ) . '/envia-shipping-module.php' );
define( 'ENVIA_HOSTNAME', 'https://ship.envia.com/' );
define( 'ENVIA_QUERIES_HOSTNAME', 'https://queries.envia.com' );
define( 'ENVIA_APP_HOSTNAME', 'https://api-clients.envia.com' );
define( 'ENVIA_LANDING', 'https://envia.com' );
define( 'ENVIA_S3', 'https://s3.us-east-2.amazonaws.com/enviapaqueteria' );
function initEnviaShipping() {
	require 'classes/envia-templates.php';
	require 'classes/envia-orders.php';
	require 'controllers/formFieldsController.php';
	require 'functions/functions.php';
	require 'functions/shippingLabels.php';
	if ( ! class_exists( 'Envia_Shipping_Module' ) ) {
		class Envia_Shipping_Module extends WC_Shipping_Method {
			private $domain_url; 
			private $active; 
			private $useLabels; 
			private $useTime; 
			private $usePickUp; 
			private $inputFormCity; 
			private $displayPickUp; 
			private $origin;
			private $origin_options;
			public function __construct( $instance_id = 0 ) {
				$this->id                 = 'envia_shipping';
				$this->instance_id        = absint( $instance_id );
				$this->title              = __( 'Live shipping rates', 'Envia.com' );
				$this->method_title       = __( 'Envia rates and shipping', 'Envia.com' );
				$this->method_description = __( 'Send easily WooCommerce orders with Envia and Rate Shipment Cost.', 'Envia.com' );
				$this->supports           = array( 'shipping-zones', 'settings', 'instance-settings', 'instance-settings-modal' );
				$this->enabled            = 'yes';

				$this->domain_url         = site_url();
				$this->origin_options     = get_option( 'woocommerce_envia_shipping_settings', array() );
				$this->active             = $this->get_option( 'active' );
				$this->useLabels          = $this->get_option( 'useLabels' );
				$this->useTime            = $this->get_option( 'useDeliveryTime' );
				$this->usePickUp          = $this->get_option( 'pickUpDestination' );
				$this->inputFormCity      = $this->get_option( 'cityType' );
				$this->displayPickUp      = $this->get_option( 'displayPickUp' );
				$this->origin  = 'Envia.com';
				// $this->availability = 'including';
				$this->init();
			}

			public function init() {
				try {
					$this->init_form_fields();
					$this->init_settings();
					add_action( 'woocommerce_update_options_shipping_' . $this->id, array( $this, 'process_admin_options' ) );
					if ( is_admin() ) {
						$cssFiles = array( 
							'configPageAdmin' => array( 
								'file' => 'config-page-admin',
								'version' => '1.3',
							),
							'configOauthAdmin' => array( 
								'file' => 'config-oauth-admin',
								'version' => '1.1',
							),
							'configOriginModal' => array( 
								'file' => 'config-origin-modal',
								'version' => '1.1',
							),
							'ordersAdmin' => array( 
								'file' => 'orders-admin',
								'version' => '1.4',
							),
						);
						$this->envia_css_loader( $cssFiles );
					}
				} catch ( Exception $err ) {
					wp_enqueue_script( 'printErrors', plugins_url( 'admin/js/errors.js', __FILE__ ), array(), '1.0', 'error' );
					wp_localize_script(
						'printErrors',
						'errorData',
						array(
							'error' => $err->getMessage(),
						)
					);
				}
			}

			public function init_form_fields() {
				$country                                = WC()->countries->get_base_country();
				$form                                   = array(
					'token'             => array(
						'title' => 'API Key',
						'type'  => 'password',
						'class' => 'envia-cfg-disabled envia-option',
					),
					'active'            => array(
						'title' => 'Envia live rate shipping',
						'type'  => 'checkbox',
						'label' => 'Display the shipping quote in your cart and checkout.',
						'class' => 'envia-option envia-top-option',
					),
					'useLabels'         => array(
						'title' => 'Carrier brand image',
						'type'  => 'checkbox',
						'label' => 'Display the rate options with carrier brand image.',
						'class' => 'envia-option',
					),
					'useDeliveryTime'   => array(
						'title' => 'Delivery time',
						'type'  => 'checkbox',
						'label' => 'Display the rate options with estimated delivery time.',
						'class' => 'envia-option',
					),
					'pickUpDestination' => array(
						'title' => 'Pick up location',
						'type'  => 'checkbox',
						'label' => 'Display pickup location options (available only in some countries).',
						'class' => 'envia-option',
					),
					'displayPickUp' => array(
						'title' => 'Pick up location format',
						'type' => 'select',
						'label' => 'Select display mode for pickup options',
						'description' => __( 'View mode for pickup locations options.', 'Tipo de aspecto para las opciones de punto de entrega.' ),
						'options'     => array(
							'standard'   => 'WooCommerce standard',
							'custom' => 'Dropdown list',
						),
						'default'     => 'standard',
						'class' => 'envia-option',
					),
					'cityType'          => array(
						'title'       => 'Input type for city',
						'type'        => 'select',
						'label'       => 'Select type of input',
						'description' => __( 'Type of input in checkout/cart/plugin page.', 'Tipo de boton en checkout/pluggin page.' ),
						'options'     => array(
							'text'   => 'Text input',
							'select' => 'Dropdown list',
						),
						'default'     => 'text',
						'class' => 'envia-cfg-disabled envia-cfg-display envia-option',
					),
				);
				$form                                   = array_merge( $form, $this->origin_address_form_fields() );
				$this->form_fields                      = $form;
				$default                                = array( 'default' => 'Default origin address' );
				$formInstance                           = $this->origin_address_form_fields();
				$formInstance['enviaOrigin']['options'] = $formInstance['enviaOrigin']['options'] ? $default + $formInstance['enviaOrigin']['options'] : $default;
				$this->instance_form_fields             = $formInstance;
			}

			public function origin_address_form_fields() {
				try {
					$form                = array();
					$originOptions       = isset( $_GET['tab'] ) && 'shipping' == $_GET['tab'] ? $this->get_envia_origin_adresses() : array();
					$form['enviaOrigin'] = array(
						'title'       => 'Select an origin address',
						'type'        => 'select',
						'label'       => 'Envia origin address',
						'description' => 'Default shipping origin address to calculate in shipping rates.',
						'options'     => $originOptions,
						'default'     => 'default',
						'class' => 'envia-option envia-top-option',
					);

					if ( $this->get_instance_id() > 0 ) {
						$form['enviaOrigin']['description'] = 'It will be your default shipping origin address to use in the shipping calculator.<br><b>Default origin address:</b> Origin address selected in <a href="admin.php?page=wc-settings&tab=shipping&section=envia_shipping" target="_blank">plugin section</a> ';
						return $form;
					}
					return $form;
				} catch ( Exception $err ) {
					throw $err;
				}
			}

			public function envia_css_loader( $files = array() ) {
				if ( 0 == count( $files ) ) {
					return 0;
				}
				foreach ( $files as $handle => $css ) {
					if ( file_exists( MDABSPATH . '/admin/css/envia-' . $css['file'] . '.css' ) ) {
						$toLoad = plugins_url( 'admin/css/envia-' . $css['file'] . '.css', __FILE__ );
						wp_enqueue_style( $handle, $toLoad, array(), $css['version'], false );
					}
				}
			}

			public function get_envia_origin_adresses() {
				try {
					if ( false == $this->get_option( 'token' ) && '' == $this->get_option( 'token' ) ) {
						return null;
					}
					$origins   = array( 'default' => 'Select a origin address' );
					$addresses = $this->requests_process( 'GET', ENVIA_QUERIES_HOSTNAME . '/shop-default-address/' . $this->get_option( 'shop' ), $this->get_option( 'token' ), null );
					foreach ( $addresses['data'] as $address ) {
						$postalCode                        = $address['postal_code'] ? ', (' . esc_html( $address['postal_code'] ) . ')' : '';
						$addressText                       = '' != $address['number'] ? esc_html( $address['street'] ) . ', #' . esc_html( $address['number'] ) . $postalCode : esc_html( $address['street'] ) . $postalCode;
						$origins[ $address['address_id'] ] = $addressText . ', ' . esc_html( $address['state'] ) . '. ' . esc_html( $address['country'] ) . '.';
					}
					return $origins;

				} catch ( Exception $e ) {
					$origins = array( '' => $e->getMessage() . ' Please refresh connection.' );
					return $origins;
				}
			}

			public function get_country_states_options( $country ) {
				$states = WC()->countries->get_states( $country ) ? WC()->countries->get_states( $country ) : $this->requests_process( 'GET', ENVIA_QUERIES_HOSTNAME . '/state?country_code=' . $country, null );
				if ( isset( $states['data'] ) ) {
					$statesAux = array();
					foreach ( $states['data'] as $state ) {
						$statesAux[ $state['code_2_digits'] ] = $state['name'];
					}
					$states = $statesAux;
				}
				return $states;
			}

			public function admin_options() {
				?>
				<div class = 'modal-envia hidden' id= 'modal-envia-oauth'>
					<div class ='saving-block' id = 'saving-oauth'>
						<div class="lds-dual-ring">
							<svg id = 'check-oauth' class="animated-check" viewBox="0 0 24 24"><path d="M4.1 12.7L9 17.6 20.3 6.3" fill="none"></path> </svg>
						</div>
						<p>Waiting to Envia.com</p>
					</div>
				</div>
				<div class='envia-top-box'>
					<div class='title-top-box'>
						<h2><?php echo esc_html( $this->method_title ); ?></h2>
						<div class='img-title'>
							<img height="30" src=<?php echo '/wp-content/plugins/' . esc_html( MAINDIR ) . '/admin/images/envia-logo-dark.svg'; ?> >
						</div>
					</div>
						<p><?php echo esc_html( $this->method_description ); ?></p>
				</div>
				<div class='envia-sub'>
					<p> Connection </p>
					<hr>
				</div>

				<div class='envia-connection'  >
					<div id='div-envia-oauth' class = 'connection-block'>
						<p>Connect your store</p>
						<input type='button' id ='envia_oauth' class='oauth-btn'>
					</div>
					<div class='connection-block'>
						<p>Connection status</p>
						<div id='status-label'>
							<div id='status-icon'></div>
							<label id='status-msj'></label>
							<!-- <a href=''>Something wrong?</a> -->
						</div>
					</div>
				</div>

				<table class="form-table" style="margin-bottom: 10px;">
					<?php $this->generate_settings_html(); ?>
				</table>
				
				<div class = 'modal-envia hidden' id= 'modal-envia'>
					<?php
						$nonce = wp_create_nonce( 'my-nonce' );
						include 'includes/forms/create-address.php';
					?>
					<div class ='saving-block' id='saving-origin'>
						<div class="lds-dual-ring">
							<svg id = 'check-origin' class="animated-check" viewBox="0 0 24 24"><path d="M4.1 12.7L9 17.6 20.3 6.3" fill="none"></path> </svg>
						</div>
						<p>Sending</p>
					</div>
				</div>
				<div>
					<div class='custom-option'>
						<p>Create a new origin address</p>
						<button type= 'button' class='custom-option-btn open-option envia-open-origin'>Add origin address</button>
					</div>
				</div>
				<div class='title-data-user'>
					<div class = 'data-user'>
						<div>
							<span>Version:</span>
							<?php echo esc_html(get_plugin_data( __FILE__ )['Version']); ?>
						</div>
						<div>
							<span>Origin:</span>
							<?php echo esc_html($this->origin); ?>
						</div>
						<div>
							<span>EnviaID:</span>
							<?php echo esc_html($this->get_option( 'company' )); ?>
						</div>
						<div>
							<span>StoreID:</span>
							<?php echo esc_html($this->get_option( 'shop' )); ?>
						</div>
					</div>
				</div>
				<?php
			}

			public function calculate_shipping( $package = array() ) {
				if ( ! ( is_cart() || is_checkout() ) ) {
					return [];
				}
			/**
			 * The changedAddress variable indicates that a pickup address has been added but new request quote is not required
			 * The refreshBlock allow know if must block the next new request quote and continue with the previous
			 */
				$changedAddress = isset( WC()->session->changed_address ) ? WC()->session->changed_address : false;
				$blockedRefresh = isset( WC()->session->blocked_refresh ) ? WC()->session->blocked_refresh : false;
				$prevRates = isset ( WC()->session->prev_envia_rates ) ? WC()->session->prev_envia_rates : null;
				if ( 'yes' == $this->active && true == get_option( 'envia_oauth_connection' ) ) {
					try {
						global $woocommerce;
						if ( ( $changedAddress || $blockedRefresh ) && ! is_null( $prevRates ) ) {
							$rates = WC()->session->get('prev_envia_rates');
							WC()->session->__unset('changed_address');
							WC()->session->__unset('blocked_refresh');
						} else {
							//Note: should be deleted the branch selected when is a new quote?
							$rates = $this->get_shipping_rates( $package );
							if ( ! is_null( $rates ) && count( $rates ) > 0 ) {
								WC()->session->set('prev_envia_rates', $rates);
							}
							if ( isset( WC()->session->last_customer_shipping_address ) ) {
								WC()->session->__unset( 'last_customer_shipping_address' );
							}
						}
						foreach ( $rates as $key => $value ) {
							if ( $value['dropOff'] > 0 && count( $value['branches'] ) == 0 ) {
								continue;
							}
							if ( ( 'no' == $this->usePickUp && count( $value['branches'] ) > 0 ) || 3 == $value['dropOff'] ) {
								continue;
							}
							if ( 2 == $value['dropOff'] ) {
								if ( 'billing_only' === get_option( 'woocommerce_ship_to_destination' ) ) {
									continue;
								}
								if ( 'standard' == $this->displayPickUp ) {
									$branchesKeys = array_keys( $value['branches'] );
									foreach ( $branchesKeys as $key ) {
										
										$rate = $this->generate_rates( $value, 2, $key);
										$this->add_rate( $rate );
									}
								}
								if ( 'custom' == $this->displayPickUp ) {
										$rate = $this->generate_rates( $value, 2, null );
										$this->add_rate( $rate );
								}
							} else {
								$rate = $this->generate_rates( $value, 0, null);
								$this->add_rate( $rate );
							}
						}
					} catch ( Exception $err ) {
						$message = $err->getCode() == 400 ? 'Enter a valid address to view shipping options' : $err->getMessage();
						wc_clear_notices();
						wc_add_notice( $message, 'notice' );
					}
				}
			}

			public function generate_rates( $value, $type, $branch = null ) {
				if ( ! is_null($branch) && 2 == $type ) {
					$addressFormat        = $value['branches'][$branch]['address']['address'] . ', #' . $value['branches'][$branch]['address']['number'] . ', (' . $value['branches'][$branch]['address']['zipcode'] . ')';
					$direction = esc_html( $value['serviceDescription'] . ' ' . $value['branches'][$branch]['address']['country'] ) . '. ' . esc_html( ucwords( $addressFormat ) );
				} else {
					$direction = esc_html( $value['serviceDescription'] );
				}
				// Branch null is a standard option
				return array(
					'id'        => ! is_null($branch) ? 'envia-' . $value['serviceId'] . '-' . $type . '-' . $value['branches'][$branch]['branch_code'] : 'envia-' . $value['serviceId'] . '-' . $type,
					'label'     => 'yes' == $this->useTime ? $direction . ' / ' . $value['deliveryEstimate'] : $direction,
					'cost'      => strval( $value['totalPrice'] ),
					'calc_tax'  => 'per_item',
					'meta_data' => array(
						'delivery'    => $value['deliveryEstimate'],
						'carrier'     => $value['carrier'],
						'dropoff'     => $value['dropOff'],
						'serviceId'     => $value['serviceId'],
						'service'     => $value['service'],
						'description' => $value['serviceDescription'],
						'branches' => is_null($branch) && 2 == $type ? $value['branches'] : null,
						'branchCode'    => ! is_null($branch) ? $value['branches'][$branch]['branch_code'] : null,
						'branchAddress'    => ! is_null($branch) ? $value['branches'][$branch]['address'] : null,
					),
				);
			}

			public function requests_process( $method, $url, $auth = null, $payload = null ) {
				if ( isset( $payload ) ) {
					$payload = json_encode( $payload );
				}
				try {
					$response = wp_remote_request(
						$url,
						array(
							'headers' => array(
								'Content-Type'  => 'application/json; charset=utf-8',
								'Authorization' => $auth,
								'Enviacom-v' => VERSION,
							),
							'body'    => $payload,
							'method'  => $method,
						)
					);
					if ( is_wp_error( $response ) ) {
						if ( array_key_exists( 'http_request_failed', $response->errors ) ) {
							throw new Exception( $response->errors['http_request_failed'][0], 408 );
						}
						$message = is_string( $response->get_error_code() ) ? $response->get_error_code() : $response->get_error_message();
						$code    = is_string( $response->get_error_code() ) ? 500 : $response->get_error_code();
						throw new Exception( $message, $code );
					} elseif ( $response['response']['code'] >= 400 ) {
						$response = json_decode( $response['body'], true );
						throw new Exception( $response['message'], $response['statusCode'] );
					} else {
						$data = json_decode( $response['body'], true );
						if ( array_key_exists( 'code', $data ) ) {
							$code = intval( $data['code'] );
							if ( $code >= 400 ) {
								throw new Exception( $data['message'], $data['code'] );
							}
						}
						return json_decode( $response['body'], true );
					}
				} catch ( Exception $err ) {
					throw $err;
				}
			}
			public function get_shipping_rates( $package ) {
				try {
					global $woocommerce;
					$url         = ENVIA_APP_HOSTNAME . '/v2/checkout/woocommerce/' . $this->get_option( 'shop' );
					$destination = $package['destination'];
					$cart        = $woocommerce->cart->get_cart();
					$origin      = $this->origin_options;
					// we can aggregate more fields like address, _1 _2 in destination
					$payload = $this->create_format_request( $origin, $destination, $cart );
					// WC()->session->set( 'last_payload_requested', $payload );
					if ( !is_null( $payload['destination']['postalCode'] ) && !is_null( $payload['items'] ) && !is_null( $payload['packages'] ) ) {
						$requestRates = $this->requests_process( 'POST', $url, null, $payload );
						if ( isset( $requestRates['meta'] ) && 'error' == $requestRates['meta'] ) {
								throw new Exception( 'Envia shipping: ' . $requestRates['error']['message'], $requestRates['error']['code'] );
						}
						for ( $i = 0; $i < count( $requestRates ); $i++ ) {
							for ( $j = 0; $j < count( $requestRates ) - 1; $j++ ) {
								if ( $requestRates[ $j + 1 ]['totalPrice'] < $requestRates[ $j ]['totalPrice'] ) {
									$rateAux                = $requestRates[ $j ];
									$requestRates[ $j ]     = $requestRates[ $j + 1 ];
									$requestRates[ $j + 1 ] = $rateAux;
								}
							}
						}
						if ( !is_null( $requestRates ) ) {					
							return $requestRates;
						} else {
							return array();
						}
					} else {
						return array();
					}
				} catch ( Exception $err ) {
					throw $err;
				}
			}
			public function create_format_request( $origin, $destination, $cart ) {
				try {
					$dimensionsOf = $this->package_dimensions( $cart );
					$payload      = array(
						'origin'      => array(
							'address_id' => $this->get_instance_option( 'enviaOrigin' ) && $this->get_instance_option( 'enviaOrigin' ) != 'default' ? $this->get_instance_option( 'enviaOrigin' ) : $origin['enviaOrigin'],
						),
						'destination' => array(
							'name'       => 'envia',
							'company'    => 'envia',
							'email'      => 'clients@envia.com',
							'phone'      => '1234567890',
							'street'     => $destination['address'],
							'number'     => $destination['address_2'],
							'district'   => $destination['address_1'],
							'city'       => $destination['city'],
							'state'      => 'CO' == $destination['country'] || 'CL' == $destination['country'] ? explode( '-', $destination['state'] )[1] : $destination['state'],
							'country'    => $destination['country'],
							'postalCode' => '' == $destination['postcode'] ? null : $destination['postcode'],
						),
						'currency'    => get_woocommerce_currency(),
						'items'       => count($dimensionsOf['items']) > 0 ? $dimensionsOf['items'] : null,
						'packages'    => count($dimensionsOf['packages']) > 0 ? $dimensionsOf['packages'] : null,
					);
					if ( ( 'CO' == $destination['country'] || 'CL' == $destination['country'] ) && '' == $destination['postcode'] ) {
						$payload['destination']['postalCode'] = '1';
					}
					return $payload;
				} catch ( Exception $err ) {
					throw $err;
				}
			}
			public function package_dimensions( $cart ) {
				try {
					$volumetricLimit = 68;
					$volumetric      = 0;
					$volumetricQuote = 0;
					$weight          = 0;
					$items           = array();
					$packages        = array();
					$lengthUnit      = get_option( 'woocommerce_dimension_unit' ) ? get_option( 'woocommerce_dimension_unit' ) : 'cm';
					$weightUnit      = get_option( 'woocommerce_weight_unit' ) ? get_option( 'woocommerce_weight_unit' ) : 'kg';
					foreach ( $cart as $element ) {
						$productId       = $element['product_id'];
						$variationId     = $element['variation_id'];
						$product         = wc_get_product( $productId );
						if ( $product->is_virtual() || $product->is_downloadable() ) {
							continue;
						}
						$height          = floatval( $product->get_height() );
						$length          = floatval( $product->get_width() );
						$width           = floatval( $product->get_length() );
						$variationWeight = 0;
						if ( $variationId > 0 ) {
							$variations = $product->get_available_variations();
							foreach ( $variations as $variation ) {
								if ( $variation['variation_id'] == $variationId ) {
									$height          = empty( floatval( $variation['dimensions']['height'] ) ) ? $height : floatval( $variation['dimensions']['height'] );
									$width           = empty( floatval( $variation['dimensions']['width'] ) ) ? $width : floatval( $variation['dimensions']['width'] );
									$length          = empty( floatval( $variation['dimensions']['length'] ) ) ? $length : floatval( $variation['dimensions']['length'] );
									$variationWeight = empty( floatval( $variation['weight'] ) ) ? 0 : floatval( $variation['weight'] );
								}
							}
						}
						$finalWeight = ( 0 != $variationWeight ? $variationWeight : ( ! empty( $product->get_weight() ) ? floatval( $product->get_weight() ) : 1 ) );
						for ( $i = 0; $i < $element['quantity']; $i++ ) {
							$volumetric     += ( $height * $width * $length ) / 5000;
							$weight         += $finalWeight;
							$volumetricQuote = $volumetric;
							$volumetricSub   = ( $height * $width * $length ) / 5000;
							if ( $volumetricQuote >= $volumetricLimit ) {
								$volumetricRate       = $volumetricQuote - $volumetricSub;
								$weight              -= $finalWeight;
								$volumetricDimensions = pow( ( $volumetricRate * 5000 ), 1 / 3 );
								$packages[]           = array(
									'content'       => 'Packages',
									'amount'        => 1,
									'type'          => 'box',
									'dimensions'    => array(
										'length' => number_format( $volumetricDimensions, 2, '.', '' ),
										'width'  => number_format( $volumetricDimensions, 2, '.', '' ),
										'height' => number_format( $volumetricDimensions, 2, '.', '' ),
									),
									'weight'        => $weight,
									'lengthUnit'    => $lengthUnit,
									'weightUnit'    => $weightUnit,
									'insurance'     => 0,
									'declaredValue' => 0,
								);
								$weight               = $finalWeight;
								$volumetric           = ( $height * $width * $length ) / 5000;
							}
						}

						$items[] = array(
							'name'               => $product->get_name(),
							'sku'                => $product->get_sku(),
							'quantity'           => $element['quantity'],
							'weight'             => $finalWeight,
							'price'              => $product->get_price(),
							'vendor'             => '',
							'requiresShipping'   => true,
							'taxable'            => true,
							'fulfillmentService' => 'manual',
							'properties'         => $height . ' * ' . $width . ' * ' . $length,
							'productId'          => $product->get_id(),
							'variantId'          => $variationId,
						);
					}
					$volumetricDimensions = pow( ( $volumetricQuote * 5000 ), 1 / 3 );
					$packages[]           = array(
						'content'       => 'Package',
						'amount'        => 1,
						'type'          => 'box',
						'dimensions'    => array(
							'length' => number_format( $volumetricDimensions, 2, '.', '' ),
							'width'  => number_format( $volumetricDimensions, 2, '.', '' ),
							'height' => number_format( $volumetricDimensions, 2, '.', '' ),
						),
						'weight'        => $weight,
						'lengthUnit'    => $lengthUnit,
						'weightUnit'    => $weightUnit,
						'insurance'     => 0,
						'declaredValue' => 0,
					);
					return array(
						'items'    => $items,
						'packages' => $packages,
					);
				} catch ( Exception $err ) {
						throw $err;
				}
			}
			public function custom_override_default_address_fields( $fields ) {
				$statePriority = $fields['shipping']['shipping_state']['priority'];
				$city_args     = wp_parse_args(
					array(
						'type'        => 'select',
						'options'     => array(
							'default' => __( 'Select city' ),
						),
						'input_class' => array(
							'country_select',
						),
						'priority'    => $statePriority + 1,
					),
					$fields['billing']['billing_city']
				);

				$fields['billing']['billing_city'] = $city_args;

				$city_args2 = wp_parse_args(
					array(
						'type'        => 'select',
						'options'     => array(
							'default' => __( 'Select city' ),
						),
						'input_class' => array(
							'country_select',
						),
						'priority'    => $statePriority + 1,
					),
					$fields['shipping']['shipping_city']
				);

				$fields['shipping']['shipping_city'] = $city_args2;

				return $fields;
			}
			public function custom_priority_address_fields( $address_fields ) {
				$statePriority                      = $address_fields['state']['priority'];
				$address_fields['city']['priority'] = $statePriority + 1;
				return $address_fields;
			}
		}
	}
}

if ( in_array( 'woocommerce/woocommerce.php', apply_filters( 'active_plugins', get_option( 'active_plugins' ) ) ) ) {
	add_action( 'plugins_loaded', 'initEnviaShipping' );
}

function add_envia_shipping_module( $methods ) {
	$methods['envia_shipping'] = 'Envia_Shipping_Module';
	return $methods;
}
add_filter( 'woocommerce_shipping_methods', 'add_envia_shipping_module' );

add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'add_plugin_page_settings_link' );

function add_plugin_page_settings_link( $links ) {
	$links += array(
		'<a href="' . admin_url( 'admin.php?page=wc-settings&tab=shipping&section=envia_shipping' ) . '">' . __( 'Settings' ) . '</a>', 
		'<a href="' . admin_url( 'admin.php?page=envia-order-manager' ) . '">' . __( 'Page' ) . '</a>' 
	);
	return $links;
}

add_action( 'before_woocommerce_init', function () {
	if ( class_exists( \Automattic\WooCommerce\Utilities\FeaturesUtil::class ) ) {
		\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', __FILE__, true );
	}
} );

ob_end_flush();
