<?php
add_action( 'admin_init', 'load_oauth' );

function load_oauth() {
	if ( is_plugin_active( MAINFILE ) ) {
		if ( ! get_option( 'envia_oauth_connection' ) ) {
			add_option( 'envia_oauth_connection', 'false' );
		}
	};
	$pages_allow_oauth = array(
		'wp-admin/plugins',
		'wp-admin/update.php?action=upload-plugin',
		'wp-admin/admin',
	);
	$self              = isset( $_SERVER['PHP_SELF'] ) ? sanitize_text_field( $_SERVER['PHP_SELF'] ) : null;
	scriptOauth(get_option( 'envia_oauth_connection' ));
}

/**
 * Call oauth.js
 */
function scriptOauth( $status ) {
	if ( 'false' == $status ) {
		update_option( 'envia_oauth_connection', 'wait' );
	}
		wp_enqueue_script( 'oauth', plugins_url( '../admin/js/oauth.js', __FILE__ ), array(), '1.9.1', false );
		wp_localize_script(
			'oauth',
			'oauthData',
			array(
				'status'  => $status,
				'dirname' => MAINDIR,
			)
		);	
}

/**
*   Callback of envia connection status
*/

/* Oauth callback check if instance variables exists */
add_action( 'wp_ajax_envia_check', 'catch_envia_status' );

/* Oauth callback create instance variables wc-api=*/
add_action( 'woocommerce_api_envia_shipping', 'catch_envia_status' );

/*Status active, inactive, cehck, update*/
function catch_envia_status() {
	if ( isset( $_GET['status'] ) ) {
			$hash      = isset( $_GET['hash'] ) ? sanitize_text_field( $_GET['hash'] ) : null;
			$shopId    = isset( $_GET['shop'] ) ? sanitize_text_field( $_GET['shop'] ) : null;
			$companyId = isset( $_GET['company'] ) ? sanitize_text_field( $_GET['company'] ) : null;
			$userId    = isset( $_GET['user'] ) ? sanitize_text_field( $_GET['user'] ) : null;
		if ( 'active' == $_GET['status'] && ! is_null( $hash ) ) {
			update_option( 'envia_oauth_connection', 'true' );
			$settings = get_option( 'woocommerce_envia_shipping_settings' );
			if ( ! isset( $settings ) ) {
				add_option(
					'woocommerce_envia_shipping_settings',
					array(
						'token'   => $hash,
						'shop'    => $shopId,
						'company' => $companyId,
						'user'    => $userId,
					)
				);
			} else {
				// if option exists but is empty
				if ( is_array( $settings ) ) {
					// if option exists with array keys but without value
					if ( isset( $settings['token'] ) ) {
						$settings['token'] = $hash;
					}
					 $settings['shop']    = $shopId;
					 $settings['company'] = $companyId;
					 $settings['user']    = $userId;
				} else {
					// if option exists totally empty
					 $settings = array(
						 'token'   => $hash,
						 'shop'    => $shopId,
						 'company' => $companyId,
						 'user'    => $userId,
					 );
				}
				update_option( 'woocommerce_envia_shipping_settings', $settings );
			}
			header( 'HTTP/1.1 201 OK' );
		} elseif ( 'inactive' == $_GET['status'] ) {
			update_option( 'envia_oauth_connection', 'false' );
			deactivate_plugins( MAINFILE );
			header( 'HTTP/1.1 200 OK' );
		} elseif ( 'check' == $_GET['status'] ) {
			$is_active = get_option( 'envia_oauth_connection' );
			$settings  = get_option( 'woocommerce_envia_shipping_settings' );
			if ( 'true' == $is_active && '' != $settings['token'] ) {
				echo json_encode(
					array(
						'status'  => 'success',
						'message' => 'Envia is connected',
					)
				);
				header( 'HTTP/1.1 201 OK' );
			} else {
				if ( 'false' == $is_active ) {
					update_option( 'envia_oauth_connection', 'wait' );
				}
				echo json_encode(
					array(
						'status'  => 'fail',
						'message' => 'Validation failed',
					)
				);
				header( 'HTTP/1.1 400' );
			}
		} elseif ( 'update' == $_GET['status'] ) {
			update_option( 'envia_oauth_connection', 'wait' );
			header( 'HTTP/1.1 200 OK' );
		} else {
			header( 'HTTP/1.1 400' );
		}
	}
	die();
}

function requests_process( $method, $url, $auth = null, $payload = null ) {
	if ( isset( $payload ) ) {
			$payload = json_encode( $payload );
	}
	try {
		$response = wp_remote_request(
			$url,
			array(
				'headers' => array(
					'Content-Type'  => 'application/json; charset=utf-8',
					'Authorization' => $auth,
					'Enviacom-v' => VERSION,
				),
				'user-agent' => 'wc-envia-plugin',
				'body'    => $payload,
				'method'  => $method,
			)
		);
		if ( is_wp_error( $response ) ) {
			if ( array_key_exists( 'http_request_failed', $response->errors ) ) {
				throw new Exception( $response->errors['http_request_failed'][0], 408 );
			}
			$message = is_string( $response->get_error_code() ) ? $response->get_error_code() : $response->get_error_message();
			$code    = is_string( $response->get_error_code() ) ? 500 : $response->get_error_code();
			throw new Exception( $message, $code );
		} elseif ( $response['response']['code'] >= 400 ) {
				$response = json_decode( $response['body'], true );
				throw new Exception( $response['message'], $response['statusCode'] );
		} else {
			$data = json_decode( $response['body'], true );
			if ( array_key_exists( 'code', $data ) ) {
				$code = intval( $data['code'] );
				if ( $code >= 400 ) {
					throw new Exception( $response['message'], $response['statusCode'] );
				}
			}
			return json_decode( $response['body'], true );
		}
	} catch ( Exception $err ) {
		throw $err;
	}
}

add_action('wp_enqueue_scripts', 'wp_enqueue_scripts_envia');

function wp_enqueue_scripts_envia() {
	wp_enqueue_style( 'enviaPublicStylesheet', plugins_url( '../public/css/envia-shipping-public.css', __FILE__ ), array(), '1.4', false );
}

/* Load assets to use in cart*/
add_action('woocommerce_after_cart', 'cart_assets_loader');
function cart_assets_loader() {
	$nonce  = ! isset( $nonce ) ? wp_create_nonce( 'my-nonce' ) : $nonce;
	wp_enqueue_script( 'pickUpDestination', plugins_url( '../public/js/pickUpDestination.js', __FILE__ ), array( 'jquery' ), '2.3', false );
	wp_localize_script(
		'pickUpDestination',
		'data',
		array(
			'viewMode'  => get_option( 'woocommerce_envia_shipping_settings' )['displayPickUp'],
			'nonce' => $nonce,
			'ratesIn' => 'cart',
		)
	);
	WC()->session->set( 'rates_in', 'cart' );
}

add_action( 'woocommerce_after_checkout_form', 'checkout_assets_loader', 20 , 1 );
function checkout_assets_loader( $checkout ) {
	$nonce  = ! isset( $nonce ) ? wp_create_nonce( 'my-nonce' ) : $nonce;
	wp_enqueue_script( 'pickUpDestination', plugins_url( '../public/js/pickUpDestination.js', __FILE__ ), array( 'jquery' ), '2.3', false );
	wp_localize_script(
		'pickUpDestination',
		'data',
		array(
			'viewMode' => get_option( 'woocommerce_envia_shipping_settings' )['displayPickUp'],
			'nonce' => $nonce,
			'ratesIn' => 'checkout',
		)
	);
	WC()->session->set( 'rates_in', 'checkout' );
}

add_action( 'woocommerce_after_shipping_rate', 'action_after_shipping_rate', 20, 2 );
function action_after_shipping_rate( $method, $index ) {
	$nonce  = ! isset( $nonce ) ? wp_create_nonce( 'my-nonce' ) : $nonce;
	$labels = get_option( 'woocommerce_envia_shipping_settings' )['useLabels'];
	if ( 'envia_shipping' == $method->method_id ) {
		if ( 'yes' == $labels ) {
			echo "<div class='rate-content'><div class= 'envia-shipping-complement-div'>" . PHP_EOL;
			echo "<img class= 'envia-shipping-carrier-img' src='https://s3.us-east-2.amazonaws.com/enviapaqueteria/uploads/logos/carriers/" . esc_html( $method->meta_data['carrier'] ) . ".svg' for= 'shipping_method_0_" . esc_html( $method->id ) . "'>" . PHP_EOL;
			echo '</div>' . PHP_EOL;
			echo '</div>' . PHP_EOL;
		}
		$methodSelected    = WC()->session->get( 'chosen_shipping_methods' )[0];
		$branchSelected = WC()->session->get( 'branch_selected' );
		$viewMode =  get_option( 'woocommerce_envia_shipping_settings' )['displayPickUp'];
		if ( ! is_null( $method->meta_data['branches'] ) && $methodSelected == $method->id && 'custom' == $viewMode ) {
			echo "<select class='pick-up-location' onChange='getCustomOptionRate(this)' value='" . esc_html( $nonce ) . "'>" . PHP_EOL;
			foreach ( $method->meta_data['branches'] as $key => $value ) {
				if ( 0 == $key ) {
					echo "<option value='none'> Select a " . esc_html( ucfirst( $method->meta_data['carrier'] ) ) . ' pick-point </option>';
					$key = -1;
				}
				$addressFormat        = $value['address']['address'] . ', #' . $value['address']['number'] . ', (' . $value['address']['zipcode'] . ')';
				if ( $branchSelected == $value['branch_code'] ) {
					echo "<option selected='selected' value='" . esc_html( $method->id . '-' . $value['branch_code']) . "'>" . esc_html( $value['address']['country'] ) . '. ' . esc_html( ucwords( $addressFormat ) ) . '</option>';
				} else {
					echo "<option value='" . esc_html( $method->id . '-' . $value['branch_code'] ) . "'>" . esc_html( $value['address']['country'] ) . '. ' . esc_html( ucwords( $addressFormat ) ) . '</option>';
				}
			}
			echo '</select>' . PHP_EOL;
		}
	}
}

add_action( 'woocommerce_applied_coupon', 'envia_coupon_action' ); 
add_action( 'woocommerce_removed_coupon', 'envia_coupon_action' );

function envia_coupon_action( $coupon_code ) {
	/**
	 * Origins that not must refresh shipping quote options
	 */	
	$ajaxOrigins = array(
		'apply_coupon',
		'remove_coupon'
	);
	if ( $coupon_code ) {
		if ( isset ( WC()->session->prev_envia_rates ) ) {
			foreach ( $ajaxOrigins as $origin ) {
				if ( isset( $_GET['wc-ajax'] ) && $origin == $_GET['wc-ajax'] ) {
					WC()->session->set( 'blocked_refresh', true );
				}
			}
		}
	}
}

add_action( 'woocommerce_checkout_order_processed', 'afterFinishOrder', 1, 1 );
function afterFinishOrder( $order_id ) {
	$lastBranch = isset( WC()->session->branch_selected ) ? WC()->session->branch_selected : null;
	if ( 'custom' == get_option( 'woocommerce_envia_shipping_settings' )['displayPickUp'] ) {
		$order = wc_get_order( $order_id );
		$shippingItemId = array_keys( $order->get_items( 'shipping' ) )[0];
		if ( ! is_null( $shippingItemId ) ) {
			$objectShipping = $order->get_item( $shippingItemId );
			if ( ! is_null( $lastBranch ) ) {
				$objectShipping->update_meta_data( 'branchCode', $lastBranch );
			}
			$objectShipping->update_meta_data( 'branches', null );
			$objectShipping->save();
			WC()->session->__unset( 'prev_envia_rates' );
			WC()->session->__unset( 'branch_selected' );
			WC()->session->__unset( 'changed_address' );
			WC()->session->__unset( 'last_customer_shipping_address' );
		}
	}
}

add_action( 'admin_menu', array( 'Envia\Templates\Loader\Envia_Templates', 'add_submenu_option' ) );
